<?php

declare(strict_types=1);

namespace app\controllers;

use yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;

use app\models\Order;

use Exception;

class CartController extends Controller
{
    /**
     * Страница: "Корзина"
     *
     * @see $this->behaviors Доступ только аутентифицированным
     */
    public function actionIndex(): string|array|null
    {
        // Инициализация
        $page = yii::$app->request->get('page') ?? yii::$app->request->post('page') ?? 1;
        $account = yii::$app->user;

        // Поиск корзины (текущего заказа)
        $model = Order::search();

        if (empty($model)) {
            // Корзина не инициализирована

            // Инициализация
            $model = new Order();
            $model->save() or throw new Exception('Не удалось инициализировать заказ');

            // Подключение
            $model->connect($account);
        }

        // Инициализация содержимого корзины
        $supplies = $model->content(10, $page);

        if (yii::$app->request->isPost) {
            // POST-запрос

            yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('index', compact('model', 'supplies')),
                'title' => 'Корзина',
                'redirect' => '/cart',
                '_csrf' => yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('index', compact('model', 'supplies'));
    }
}
